% layout.pl
% Author: Silvio do Lago Pereira

:- module(layout, [layout/2]).

:- dynamic node/5.

%font_query(font(times,oblique,13)).
%font_mgu(font(times,oblique,11)).
%font_leaf(font(times,oblique,13)).

font_query(font(screen,normal,15)).
font_mgu(font(screen,normal,13)).
font_leaf(font(screen,bold,15)).
x_gap(10).
y_gap(50).

layout(Tree,NewTree) :-
    retractall(node(_,_,_,_,_)),
    forall((current_flag(F), integer(F)), flag(F, _, 0)),
    forall(member(Node, Tree), new_node(Node)),
    layout,
    y_gap(Ygap),
    findall(node(Node,Text,X,Y),
           ( retract(node(Node,_,Text,X,Level)),
             Y is Ygap*Level),
           NewTree).

new_node(node(Node,Father,Atom,Mode,Type,X,Y)) :-
    new_text(Mode,Atom,Text,Type),
    assertz(node(Node,Father,Text,X,Y)),
    (   Father>0
    ->  node(Father,_,FatherText,_,_),
        new_link(Mode,FatherText,Text,Type)
    ;   true).

new_text(Mode,Atom,Text,Type) :-
    new(Text,text(Atom)),
    font_query(FQ),
    font_mgu(FM),
    memberchk(Type-Attributes,
              [ second - [ font(FQ), %font(screen,normal,15)),
                           colour(colour(blue)),
                           background(oldlace),
                           border(3),
                           handle(handle(w/2,0,up)),
                           handle(handle(w/2,h,dn))], % recogniser(new(move_gesture(left)))
               none    - [ font(FM), %font(screen,normal,13)),
                           colour(colour(black)),
                           border(3),
                           handle(handle(w/2,0,up)),
                           handle(handle(w/2,h,dn))] ]), % recogniser(new(move_gesture(left)))
    send_list(Text,Attributes),
    (   Type = second
    ->  (   Mode = any
        ->  send(Text,colour,darkred),
            send(Text,background,lavenderblush1)
        ;   send(Text,colour,darkblue) )
    ;   true ).


new_link(Mode,Father,Child,Arrow) :-
    new(Link,link(dn,up,new(Line,line(arrows:=Arrow)))),
    ( Mode = any
    ->  send(Line,texture,dotted), %dashed
        send(Line,colour,indianred) %lightpink3) %salmon)
    ;   send(Line,colour,cadetblue4) ),
    send(Father,connect,Child,Link).

layout :-
    adjust(1, 0),
    font_leaf(FL),
    forall(  leaf(Text),
	     (  get(Text, value, Value),
                (  Value = true  -> send(Text, colour, blue)
                ;  Value = false -> send(Text, colour, red)
                ;  Value = cut   -> send(Text, colour, chocolate1)),
                send(Text, attribute, boxed, @on),
                send(Text, font, FL)) ).

adjust(Node,Y) :-
    Yc is Y+1,
    forall(node(Child,Node,_,_,_),adjust(Child,Yc)),
    flag(Y,X,X),
    median(Node,Xm),
    Xn is max(X,Xm),
    setpos(Node,Xn,Y),
    Dx is Xn-Xm,
    forall((Dx>0, node(Child,Node,_,_,_)), move(Child, Dx)).

move(Node,Dx) :-
    (   node(Node,_,_,X,Y)
    ->  setpos(Node,X+Dx,Y),
        forall(node(Child,Node,_,_,_),move(Child,Dx))
    ;   true ).

median(Node,Xm) :-
    findall(X,node(_,Node,_,X,_),Xs),
    (   Xs = []
    ->  Xm is 0
    ;   min_list(Xs,Xmin),
        max_list(Xs,Xmax),
        node(Node,_,NodeText,_,_),
        node(_,Node,ChildText,Xmax,_),
        get(NodeText,width,Wn),
        get(ChildText,width,Wc),
        Xm is (Xmin+Xmax+Wc)/2 - Wn/2 ).

setpos(N,E,Y) :-
    X is E,
    retract(node(N,F,T,_,_)),
    assertz(node(N,F,T,X,Y)),
    get(T,width,W),
    x_gap(Xgap),
    Xn is X+W+Xgap,
    flag(Y,_,Xn), !.

leaf(Text) :-
    node(_,_,Text,_,_),
    get(Text,value,Value),
    memberchk(Value,[true,false,cut]).





