% varname.pl
% Author: Silvio do Lago Pereira

:- module(varname,
          [ reset_varname/0
          , get_varname/2
          , namevars/2
          , varcmp/3
          , pretty_atom/2]).

% Reset varname counter

reset_varname :-
    flag(vn, _, 0).

% Get/set varname

get_varname(Var, Name) :-
    (   var(Var)
    ->  (   get_attr(Var, varname, Name)
        ->  true
        ;   new_name(Name),
            put_attr(Var, varname, Name) )
    ;   true ).

% Name variables in a term

namevars(T, N) :-
    atom(T),
    T = '', !,
    N = ''.

namevars(T, N) :-
    atom(T), !,
    atom_chars(T, [C|Cs]),
    (  (  char_type(C, upper)
       ;  member(X, Cs), not(char_type(X,csym)) )
    -> format(atom(N), '\'~w\'', [T])
    ;  format(atom(N), '~w', [T]) ).

namevars(T, T) :-
    atomic(T), !.

namevars(T, N) :-
    var(T), !,
    get_varname(T, N).

namevars(T1, T2) :-
    T1 =.. [Functor|Arguments],
    (   (  Functor = '[|]'
        ;  current_op(_,  _, Functor))
    ->  NamedFunctor = Functor
    ;   namevars(Functor, NamedFunctor) ),
    namevars_args(Arguments, NamedArguments),
    T2 =.. [NamedFunctor|NamedArguments].

% Compare variable names

varcmp(=, N, N) :- !.

varcmp(Result, A, B) :-
    split(A, AL, AN),
    split(B, BL, BN),
    (  ( AL @< BL ;
         AL = BL, AN < BN )
       -> Result = '<'
       ;  Result = '>' ).

% Convert term to pretty atom

pretty_atom(Term, Atom) :-
    namevars(Term, NamedTerm),
    format(atom(Atom), '~w', [NamedTerm]).

% Value N has been assigned to an attributed variable V with attribute M

attr_unify_hook(N, V) :-
    (   var(V)
    ->  get_attr(V, varname, M),
        (  varcmp(<, N, M)
        -> put_attr(V, varname, N)
        ;  true )
    ;   true ).

% Translate attributes from this module to residual goals

attribute_goals(Var) -->
        { get_attr(Var, varname, Name) },
        [ varname(Var, Name) ].

% Auxiliary predicates

namevars_args([], []).
namevars_args([T|Ts], [N|Ns]) :-
    namevars(T, N),
    namevars_args(Ts, Ns).

new_name(Name) :-
    flag(vn, N, N+1),
    M is (N mod 26) + 65,
    name(O, [M]),
    (  N < 26
    ->  Name = O
    ;   gensym(O, Name) ).

split(Name, Letter, Number) :-
    atom_chars(Name, [Letter|Rest]),
    atomic_list_concat(Rest, Atom),
    (  Atom = ''
    -> Number = 0
    ;  atom_number(Atom, Number) ).
